package net.sf.doolin.bus.ext.array;

import java.util.Collections;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.util.Utils;
import net.sf.sido.array.Array;
import net.sf.sido.array.ArrayEvent;
import net.sf.sido.array.ArrayEventElement;
import net.sf.sido.array.ArrayEventType;
import ca.odell.glazedlists.AbstractEventList;
import ca.odell.glazedlists.util.concurrent.LockFactory;

public class ArrayEventList<E> extends AbstractEventList<E> {

	private final Object parent;
	private final String property;

	public ArrayEventList(SubscriberValidator validator, Object parent,
			String property) {
		this.parent = parent;
		this.property = property;
		// Event list properties
		readWriteLock = LockFactory.DEFAULT.createReadWriteLock();
		// Listens for changes on the array
		PropertyChangeSupport.subscribe(validator, parent, property,
				new Runnable() {

					@Override
					public void run() {
						fireListChange();
					}
				});
	}

	@Override
	public void add(int index, E value) {
		// Gets the initial array
		Array<E> array = getArray();
		// Adds the value to the array
		array = array.insert(index, Collections.singletonList(value));
		// Sets the array
		setArray(array);
	}

	@Override
	public void dispose() {
	}

	protected void fireListChange() {
		// Gets the array event
		Array<E> array = getArray();
		ArrayEvent<E> arrayEvent = array.getArrayEvent();
		// Collects the EventList events
		updates.beginEvent();
		for (ArrayEventElement<E> eventElement : arrayEvent) {
			ArrayEventType type = eventElement.getType();
			int index = eventElement.getIndex();
			E value = eventElement.getValue();
			if (type == ArrayEventType.ADDED) {
				updates.elementInserted(index, value);
			} else if (type == ArrayEventType.UPDATED) {
				updates.elementUpdated(index, value, array.get(index));
			} else if (type == ArrayEventType.DELETED) {
				updates.elementDeleted(index, value);
			} else {
				throw new IllegalStateException("Unknown ArrayEventType: "
						+ type);
			}
		}
		// Fires the event
		updates.commitEvent();
	}

	@Override
	public E get(int index) {
		return getArray().get(index);
	}

	@SuppressWarnings("unchecked")
	protected Array<E> getArray() {
		return (Array<E>) Utils.getProperty(this.parent, this.property);
	}

	public Object getParent() {
		return this.parent;
	}

	public String getProperty() {
		return this.property;
	}

	@Override
	public E remove(int index) {
		// Gets the initial array
		Array<E> array = getArray();
		// Old value
		E old = array.get(index);
		// Deletes the value
		array = array.delete(index);
		// Sets the array
		setArray(array);
		// Old value
		return old;
	}

	@Override
	public E set(int index, E value) {
		// Gets the initial array
		Array<E> array = getArray();
		// Old value
		E old = array.get(index);
		// Updates the value
		array = array.update(index, value);
		// Sets the array
		setArray(array);
		// Old value
		return old;
	}

	protected void setArray(Array<E> array) {
		Utils.setProperty(this.parent, this.property, array);
	}

	@Override
	public int size() {
		return getArray().length();
	}

}
